import { ChangeDetectorRef, ComponentFactoryResolver, Directive, ElementRef, EventEmitter, forwardRef, HostListener, Injector, Input, Output, Renderer2, ViewContainerRef } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { fromEvent } from 'rxjs/observable/fromEvent';
import { of } from 'rxjs/observable/of';
import 'rxjs/add/operator/map';
import 'rxjs/add/operator/filter';
import 'rxjs/add/operator/debounceTime';
import 'rxjs/add/operator/distinctUntilChanged';
import 'rxjs/add/operator/switchMap';
import 'rxjs/add/operator/do';
import { RebirthNGConfig } from '../rebirth-ng.config';
import { PositionService } from '../position/positioning.service';
import { AutoCompletePopupComponent } from './auto-complete-popup.component';
import { stopPropagationIfExist } from '../utils/dom-utils';
var AutoCompleteDirective = /** @class */ (function () {
    function AutoCompleteDirective(elementRef, viewContainerRef, componentFactoryResolver, renderer, injector, positionService, rebirthNGConfig, changeDetectorRef) {
        this.elementRef = elementRef;
        this.viewContainerRef = viewContainerRef;
        this.componentFactoryResolver = componentFactoryResolver;
        this.renderer = renderer;
        this.injector = injector;
        this.positionService = positionService;
        this.rebirthNGConfig = rebirthNGConfig;
        this.changeDetectorRef = changeDetectorRef;
        this.appendBody = false;
        this.selectValueChange = new EventEmitter();
        this.placement = 'bottom-left';
        this.onChange = function (_) { return null; };
        this.onTouched = function () { return null; };
        this.valueChanges = this.registerInputEvent(elementRef);
        this.delay = rebirthNGConfig.autoComplete.delay;
        this.minLength = rebirthNGConfig.autoComplete.minLength;
        this.itemTemplate = rebirthNGConfig.autoComplete.itemTemplate;
        this.noResultItemTemplate = rebirthNGConfig.autoComplete.noResultItemTemplate;
        this.formatter = rebirthNGConfig.autoComplete.formatter;
        this.valueParser = rebirthNGConfig.autoComplete.valueParser;
    }
    AutoCompleteDirective.prototype.ngOnInit = function () {
        var _this = this;
        this.subscription = this.valueChanges
            .subscribe(function (source) { return _this.onSourceChange(source); });
        var factory = this.componentFactoryResolver.resolveComponentFactory(AutoCompletePopupComponent);
        var viewContainerRef = this.appendBody ? this.rebirthNGConfig.rootContainer : this.viewContainerRef;
        // EXCEPTION: Expression has changed after it was checked when append to body;
        setTimeout(function () {
            _this.popupRef = viewContainerRef.createComponent(factory, _this.viewContainerRef.length, _this.injector);
            _this.fillPopup();
            _this.positionPopup();
            _this.popupRef.instance.registerOnChange(function (item) {
                var value = _this.valueParser(item);
                _this.writeValue(value);
                _this.onChange(value);
                _this.hidePopup();
                _this.selectValueChange.emit(item);
            });
        }, 0);
    };
    Object.defineProperty(AutoCompleteDirective.prototype, "dataSource", {
        set: function (dataSource) {
            if (dataSource) {
                this.arraySource = dataSource;
                this.setupArraySource();
            }
        },
        enumerable: true,
        configurable: true
    });
    AutoCompleteDirective.prototype.setupArraySource = function () {
        var _this = this;
        if (this.popupRef) {
            this.popupRef.instance.source = this.arraySource;
        }
        this.onSearch = function (term) {
            return of(_this.arraySource
                .filter(function (item) { return _this.formatter(item).toLowerCase().indexOf((term || '').toLowerCase()) !== -1; }));
        };
    };
    AutoCompleteDirective.prototype.writeValue = function (obj) {
        this.value = obj || '';
        this.writeInputValue(this.value);
    };
    AutoCompleteDirective.prototype.registerOnChange = function (fn) {
        this.onChange = fn;
    };
    AutoCompleteDirective.prototype.registerOnTouched = function (fn) {
        this.onTouched = fn;
    };
    AutoCompleteDirective.prototype.setDisabledState = function (isDisabled) {
        this.disabled = isDisabled;
        this.renderer.setProperty(this.elementRef.nativeElement, 'disabled', isDisabled);
        if (this.popupRef) {
            this.popupRef.instance.setDisabledState(isDisabled);
        }
    };
    AutoCompleteDirective.prototype.ngOnDestroy = function () {
        this.unSubscription();
        this.removePopView();
    };
    AutoCompleteDirective.prototype.removePopView = function () {
        if (this.popupRef) {
            this.popupRef.destroy();
        }
    };
    AutoCompleteDirective.prototype.onBlur = function () {
        this.onTouched();
    };
    AutoCompleteDirective.prototype.onEscKeyup = function ($event) {
        this.hidePopup();
    };
    AutoCompleteDirective.prototype.onEnterKeyDown = function ($event) {
        if (this.popupRef && this.popupRef.instance.isOpen) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.selectCurrentItem();
            this.hidePopup();
        }
    };
    AutoCompleteDirective.prototype.onArrowUpKeyDown = function ($event) {
        if (this.popupRef) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.prev();
        }
    };
    AutoCompleteDirective.prototype.onArrowDownKeyDown = function ($event) {
        if (this.popupRef) {
            $event.preventDefault();
            $event.stopPropagation();
            this.popupRef.instance.next();
        }
    };
    AutoCompleteDirective.prototype.onDocumentClick = function ($event) {
        var hostElement = this.elementRef.nativeElement;
        if ($event.target !== hostElement) {
            this.hidePopup();
        }
    };
    AutoCompleteDirective.prototype.onSourceChange = function (source, term) {
        if (term !== undefined) {
            this.value = term;
        }
        var pop = this.popupRef.instance;
        pop.reset();
        this.fillPopup(source, this.value);
        if ((source && source.length) || this.noResultItemTemplate) {
            pop.show();
            this.positionPopup();
            this.changeDetectorRef.markForCheck();
        }
    };
    AutoCompleteDirective.prototype.hidePopup = function () {
        if (this.popupRef) {
            this.popupRef.instance.activeIndex = 0;
            this.popupRef.instance.hide();
        }
    };
    AutoCompleteDirective.prototype.toggle = function ($event) {
        stopPropagationIfExist($event);
        if (this.popupRef) {
            var pop = this.popupRef.instance;
            if (pop.isOpen) {
                this.hidePopup();
                return;
            }
            this.onSourceChange(this.arraySource);
        }
    };
    AutoCompleteDirective.prototype.positionPopup = function () {
        var targetElement = this.popupRef.location.nativeElement;
        var hostElement = this.placementElement || this.elementRef.nativeElement;
        var clientRect = this.positionService.positionElements(hostElement, targetElement, this.placement, this.appendBody);
        this.renderer.setStyle(targetElement, 'left', clientRect.left + "px");
        this.renderer.setStyle(targetElement, 'top', clientRect.top + "px");
    };
    AutoCompleteDirective.prototype.fillPopup = function (source, term) {
        var _this = this;
        var pop = this.popupRef.instance;
        pop.source = source;
        pop.term = term;
        ['formatter', 'itemTemplate', 'noResultItemTemplate', 'cssClass']
            .forEach(function (key) {
            if (_this[key] !== undefined) {
                pop[key] = _this[key];
            }
        });
    };
    AutoCompleteDirective.prototype.writeInputValue = function (value) {
        this.renderer.setProperty(this.elementRef.nativeElement, 'value', this.formatter(value || '') || '');
    };
    AutoCompleteDirective.prototype.unSubscription = function () {
        if (this.subscription) {
            this.subscription.unsubscribe();
            this.subscription = null;
        }
    };
    AutoCompleteDirective.prototype.onTermChange = function (term) {
        this.value = term;
        if (this.popupRef) {
            this.popupRef.instance.term = term;
        }
        this.onChange(term);
    };
    AutoCompleteDirective.prototype.registerInputEvent = function (elementRef) {
        var _this = this;
        return fromEvent(elementRef.nativeElement, 'input')
            .map(function (e) { return e.target.value; })
            .do(function (term) { return _this.onTouched(); })
            .filter(function (term) { return !_this.disabled && _this.onSearch && term.length >= _this.minLength; })
            .debounceTime(this.delay)
            .do(function (term) { return _this.onTermChange(term); })
            .switchMap(function (term) { return _this.onSearch(term, _this); });
    };
    AutoCompleteDirective.decorators = [
        { type: Directive, args: [{
                    selector: '[reAutoComplete]',
                    exportAs: 'autoComplete',
                    host: {
                        'autocomplete': 'off',
                        'autocapitalize': 'off',
                        'autocorrect': 'off'
                    },
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(function () { return AutoCompleteDirective; }),
                            multi: true
                        }]
                },] },
    ];
    /** @nocollapse */
    AutoCompleteDirective.ctorParameters = function () { return [
        { type: ElementRef, },
        { type: ViewContainerRef, },
        { type: ComponentFactoryResolver, },
        { type: Renderer2, },
        { type: Injector, },
        { type: PositionService, },
        { type: RebirthNGConfig, },
        { type: ChangeDetectorRef, },
    ]; };
    AutoCompleteDirective.propDecorators = {
        'disabled': [{ type: Input },],
        'cssClass': [{ type: Input },],
        'delay': [{ type: Input },],
        'minLength': [{ type: Input },],
        'appendBody': [{ type: Input },],
        'itemTemplate': [{ type: Input },],
        'noResultItemTemplate': [{ type: Input },],
        'formatter': [{ type: Input },],
        'valueParser': [{ type: Input },],
        'onSearch': [{ type: Input },],
        'selectValueChange': [{ type: Output },],
        'placementElement': [{ type: Input },],
        'dataSource': [{ type: Input },],
        'onBlur': [{ type: HostListener, args: ['blur', [],] },],
        'onEscKeyup': [{ type: HostListener, args: ['keydown.esc', ['$event'],] },],
        'onEnterKeyDown': [{ type: HostListener, args: ['keydown.Enter', ['$event'],] },],
        'onArrowUpKeyDown': [{ type: HostListener, args: ['keydown.ArrowUp', ['$event'],] },],
        'onArrowDownKeyDown': [{ type: HostListener, args: ['keydown.ArrowDown', ['$event'],] },],
        'onDocumentClick': [{ type: HostListener, args: ['document:click', ['$event'],] },],
    };
    return AutoCompleteDirective;
}());
export { AutoCompleteDirective };
//# sourceMappingURL=auto-complete.directive.js.map